@@ -11,8 +11,10 @@ This module implements the HTTP part of the API backend of
 IntelMQ-Manager. The logic itself is in the runctl & files modules.
 """
 
+import json
 import sys
 import os
+import pathlib
 import string
 import typing
 
@@ -23,6 +25,7 @@ import intelmq_api.files as files
 import intelmq_api.config
 import intelmq_api.session as session
 
+from intelmq.lib import utils  # type: ignore
 
 Levels = hug.types.OneOf(["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL",
                           "ALL"])
@@ -73,7 +76,6 @@ def initialize_api(config: intelmq_api.config.Config) -> None:
 
     runner = runctl.RunIntelMQCtl(api_config.intelmq_ctl_cmd)
     file_access = files.FileAccess(api_config)
-    file_access.update_from_runctl(runner.get_paths())
 
     session_file = api_config.session_store
     if session_file is not None:
@@ -136,6 +138,11 @@ def queues_and_status():
     return runner.list("queues-and-status")
 
 
+@cache_get("/api/bots", requires=token_authentication, versions=1)
+def bots():
+    return runner.list("bots")
+
+
 @hug.get("/api/version", requires=token_authentication, versions=1)
 def version(request):
     return runner.version()
@@ -176,25 +183,72 @@ def config(response, file: str, fetch: bool=False):
     return contents
 
 
-@hug.post("/api/save", parse_body=True,
-          inputs={"application/x-www-form-urlencoded": hug.input_format.text},
-          requires=token_authentication, versions=1)
-def save(body, file: str):
-    global file_access
-    try:
-        file_access.save_file(file, body)
-        return "success"
-    except files.SaveFileException as e:
-        return str(e)
-
-
 @hug.post("/api/login", versions=1)
 def login(username: str, password: str):
-    if session_store is not None:
+    if session_store is None:
+        return {"error": "Session store is disabled by configuration! No login possible and required."}
+    else:
         known = session_store.verify_user(username, password)
         if known is not None:
             token = session_store.new_session({"username": username})
             return {"login_token": token,
                     "username": username,
                     }
-    return "Invalid username and/or password"
+        else:
+            return {"error": "Invalid username and/or password."}
+
+
+@hug.get("/api/harmonization", requires=token_authentication, versions=1)
+def get_harmonization():
+    harmonization = pathlib.Path('/opt/intelmq/etc/harmonization.conf')
+    paths = runner.get_paths()
+    if 'CONFIG_DIR' in paths:
+        harmonization = pathlib.Path(paths['CONFIG_DIR']) / 'harmonization.conf'
+    try:
+        return json.loads(harmonization.read_text())
+    except OSError as e:
+        print(f"Could not read {harmonization}: {str(e)}")
+        return {}
+
+
+@hug.get("/api/runtime", requires=token_authentication, versions=1)
+def get_runtime():
+    return utils.get_runtime()
+
+
+@hug.post("/api/runtime", requires=token_authentication, version=1)
+def post_runtime(body):
+    try:
+        utils.set_runtime(body)
+        return "success"
+    except Exception as e:
+        print(f"Could not write runtime {str(e)}")
+        return str(e)
+
+
+@hug.get("/api/positions", requires=token_authentication, versions=1)
+def get_positions():
+    positions = pathlib.Path('/opt/intelmq/etc/manager/positions.conf')
+    paths = runner.get_paths()
+    if 'CONFIG_DIR' in paths:
+        positions = pathlib.Path(paths['CONFIG_DIR']) / 'manager/positions.conf'
+    try:
+        return json.loads(positions.read_text())
+    except OSError as e:
+        print(f"Could not read {positions}: {str(e)}")
+        return {}
+
+
+@hug.post("/api/positions", requires=token_authentication, version=1)
+def post_positions(body):
+    positions = pathlib.Path('/opt/intelmq/etc/manager/positions.conf')
+    paths = runner.get_paths()
+    if 'CONFIG_DIR' in paths:
+        positions = pathlib.Path(paths['CONFIG_DIR']) / 'manager/positions.conf'
+    try:
+        positions.parent.mkdir(exist_ok=True)
+        positions.write_text(json.dumps(body, indent=4))
+        return "success"
+    except OSError as e:
+        print(f"Error creating {positions.parent} or writing to {positions}: {str(e)}")
+        return str(e)
